#ifndef cxxtls_AutoDestroyingPointer_header_included_p
#define cxxtls_AutoDestroyingPointer_header_included_p

template<class T>
struct AutoDestroyingPointer
    /// a templated pointer wrapper that models auto_ptr but which uses a member function,
    /// destroy() rather than delete to destruct.
{
     T *value_;

     AutoDestroyingPointer()
     :  value_(0)
     {
     }

     AutoDestroyingPointer(T *t)
     :  value_(t)
     {
     }

     ~AutoDestroyingPointer()
     {
        if(value_)
            T::destroy(value_);
     }

     T*        get()       { return value_; }
     T const * get() const { return value_; }

     void reset(T* value)
     {
        if(value_)
            T::destroy(value_);

        value_ = value;
     }

     void release()
        // assume we have given the pointer away to someone else who will delete it.
     {
        value_ = 0;  
     }

     T*          operator->()       { return value_; }
     T const *   operator->() const { return value_; }

private:
     AutoDestroyingPointer(AutoDestroyingPointer const &);             // not implement
     AutoDestroyingPointer operator=(AutoDestroyingPointer const &);   // not implemented
};

#endif
